import obt.env
import obt.host
import obt.path
import obt.deco
import obt.command
import os, sys, re, json

deco = obt.deco.Deco()


##########################################

def root_path():
  import obt.path
  if obt.path.running_from_pip():
    return obt.path.obt_data_base()
  elif "OBT_ROOT" in os.environ:
    return obt.path.Path(os.environ["OBT_ROOT"])
  else:
    return None

class EnvSetup:
  def __init__(self,stagedir=None,
                    rootdir=None,
                    projectdir=None,
                    bindir=None,
                    scriptsdir=None,
                    disable_syspypath=False,
                    is_quiet=False,
                    project_name=None,
                    git_ssh_command=None):
    
    if stagedir==None:
      stagedir = obt.path.Path(os.environ["OBT_STAGE"])
    if rootdir==None:
      rootdir = root_path()
    if bindir==None:
      bindir = rootdir/"bin"
    if projectdir==None:
      projectdir = obt.path.Path(os.environ["OBT_PROJECT_DIR"])
    try_project_manifest = projectdir/"obt.project"/"obt.manifest"
    if try_project_manifest.exists():
      try_project_bin = projectdir/"obt.project"/"bin"
      if try_project_bin.exists():
        obt.env.append("PATH",try_project_bin)

    if scriptsdir==None:
      scriptsdir = rootdir/"scripts"
    if project_name==None and "OBT_PROJECT_NAME" in os.environ:
      project_name = os.environ["OBT_PROJECT_NAME"] 
    if git_ssh_command==None and "OBT_GIT_SSH_COMMAND" in os.environ:
      git_ssh_command = os.environ["OBT_GIT_SSH_COMMAND"] 

    self.OBT_STAGE = stagedir 
    self.ROOT_DIR = rootdir 
    self.PROJECT_DIR = projectdir 
    self.BIN_DIR = bindir 
    self.SCRIPTS_DIR = scriptsdir 
    self.DISABLE_SYSPYPATH=disable_syspypath
    self.IS_QUIET = is_quiet
    self.PROJECT_NAME = project_name
    self.GIT_SSH_COMMAND = git_ssh_command

  ##########################################
  def install(self):
    import obt.path
    orig_path = ""
    orig_ld_library_path = ""
    orig_ps1 = ""
    orig_pkg_config = ""
    orig_pkg_config_path = ""
    orig_python_path = ""
    if "PATH" in os.environ:
      orig_path = os.environ["PATH"]
    if "PS1" in os.environ:
      orig_ps1 = os.environ["PS1"]
    if "LD_LIBRARY_PATH" in os.environ:
      orig_ld_library_path = os.environ["LD_LIBRARY_PATH"]

    ##############################################################################
    # retrieve original PKG_CONFIG_PATH
    ##############################################################################

    orig_pkg_config_path = obt.command.capture(["pkg-config","--variable","pc_path","pkg-config"])
    orig_pkg_config_path = orig_pkg_config_path.replace("\n","")
    print("orig_pkg_config_path<%s>"%orig_pkg_config_path)

    if "PKG_CONFIG" in os.environ:
      orig_pkg_config = os.environ["PKG_CONFIG"]
    #if "PKG_CONFIG_PATH" in os.environ:
    #  orig_pkg_config_path = os.environ["PKG_CONFIG_PATH"]
    if "PYTHONPATH" in os.environ:
      orig_python_path = os.environ["PYTHONPATH"]

    obt.env.set("PKG_CONFIG_PATH",orig_pkg_config_path )
    obt.env.set("OBT_ORIGINAL_PKG_CONFIG_PATH",orig_pkg_config_path )

    obt.env.set("OBT_ORIGINAL_PKG_CONFIG",orig_pkg_config )

    ##############################################################################

    obt.env.set("color_prompt","yes")
    obt.env.set("OBT_STAGE",self.OBT_STAGE)
    obt.env.set("OBT_BUILDS",self.OBT_STAGE/"builds")
    obt.env.set("OBT_ROOT",self.ROOT_DIR)
    obt.env.set("OBT_PROJECT_DIR",self.PROJECT_DIR)
    obt.env.set("OBT_SUBSPACE","host")
    obt.env.set("OBT_SUBSPACE_PROMPT","host")
    obt.env.set("OBT_SUBSPACE_DIR",self.OBT_STAGE)
    obt.env.set("OBT_PROJECT_NAME",self.PROJECT_NAME)
    obt.env.set("OBT_ORIGINAL_PATH",orig_path )
    obt.env.set("OBT_ORIGINAL_LD_LIBRARY_PATH",orig_ld_library_path )
    obt.env.set("OBT_ORIGINAL_PS1",orig_ps1 )
    obt.env.set("OBT_ORIGINAL_PYTHONPATH",orig_python_path )
    obt.env.set("OBT_SCRIPTS_DIR",self.SCRIPTS_DIR )
    obt.env.set("OBT_PYTHONHOME",self.OBT_STAGE/"pyvenv")
    obt.env.set("OBT_SUBSPACE_LIB_DIR",obt.path.libs())
    obt.env.set("OBT_SUBSPACE_BIN_DIR",obt.path.bin())
    obt.env.prepend("PATH",self.BIN_DIR )
    obt.env.prepend("PATH",self.OBT_STAGE/"bin")
    obt.env.prepend("LD_LIBRARY_PATH",self.OBT_STAGE/"lib")
    obt.env.prepend("LD_LIBRARY_PATH",self.OBT_STAGE/"lib64")

    obt.env.append("OBT_MODULES_PATH",obt.path.modules())
    obt.env.append("OBT_DEP_PATH",obt.path.modules()/"dep")

    if self.GIT_SSH_COMMAND!=None:
      obt.env.set("OBT_GIT_SSH_COMMAND",self.GIT_SSH_COMMAND)

    obt_prj_extensions = self.PROJECT_DIR/"obt.project"
    print(self.PROJECT_DIR)
    if obt_prj_extensions.exists():
      self.importProject(obt_prj_extensions)


    #if obt.host.IsLinux:

      #if obt.host.IsDebian:
      #  pkgcfgdir = obt.path.Path("/lib/x86_64-linux-gnu/pkgconfig")
      #elif obt.host.IsGentoo:
      #  pkgcfgdir = obt.path.Path("/usr/lib64/pkgconfig")
      #elif obt.host.IsAARCH64:
      #  pkgcfgdir = obt.path.Path("/usr/lib/pkgconfig")

      #if pkgcfgdir.exists():
      #  obt.env.append("PKG_CONFIG_PATH",pkgcfgdir)
      #pkgcfgdir = obt.path.Path("/usr/share/pkgconfig")
      #if pkgcfgdir.exists():
      #  obt.env.append("PKG_CONFIG_PATH",pkgcfgdir)
    #elif obt.host.IsDarwin:
      #pkgcfgdir = obt.path.Path("/usr/local/lib/pkgconfig")
      #if pkgcfgdir.exists():
      #  obt.env.append("PKG_CONFIG_PATH",pkgcfgdir)


    if obt.path.vivado_base().exists():
        obt.env.append("PATH",obt.path.vivado_base()/"bin")
    
    #####################################
    # Python Env Init
    #####################################
    
    if not obt.host.IsAARCH64:
      PYTHON = obt.dep.instance("python")
    
    #####################################
    # Late init
    #####################################
    obt.env.prepend("PKG_CONFIG",self.OBT_STAGE/"bin"/"pkg-config")
    #obt.env.prepend("PKG_CONFIG_PREFIX",self.OBT_STAGE)
    obt.env.prepend("PKG_CONFIG_PATH",self.OBT_STAGE/"lib"/"pkgconfig")
    obt.env.prepend("PKG_CONFIG_PATH",self.OBT_STAGE/"lib64"/"pkgconfig")

    if True: # WIP
      obt.env.set("PYTHONNOUSERSITE","TRUE")
      obt.env.append("PYTHONPATH",self.SCRIPTS_DIR)
      obt.env.prepend("PYTHONPATH",self.OBT_STAGE/"lib"/"python")
      obt.env.append("LD_LIBRARY_PATH",self.OBT_STAGE/"python-3.9.13"/"lib")

    if obt.path.running_from_pip():
      obt.env.prepend("PATH",obt.path.obt_data_base()/"bin_priv")

    
  ###########################################

  def importProject(self,prjdir):
    init_script = prjdir/"scripts"/"obt.env.extension.py"
    #print(init_script)
    if init_script.exists():
      import importlib
      modulename = importlib.machinery.SourceFileLoader('modulename',str(init_script)).load_module()
      #print(modulename)
      modulename.setup()
      #modul.setup()
    modules_dir = prjdir/"modules"
    #print(modules_dir,modules_dir.exists())
    if modules_dir.exists():
      obt.env.prepend("OBT_MODULES_PATH",modules_dir)

  ###########################################
  def log(self,x):
    if not self.IS_QUIET:
       print(x)
  ###########################################
  def lazyMakeDirs(self):
    self.log(deco.bright("Making required directories"))
    (obt.path.prefix()/"lib").mkdir(parents=True,exist_ok=True)
    (obt.path.prefix()/"bin").mkdir(parents=True,exist_ok=True)
    (obt.path.prefix()/"include").mkdir(parents=True,exist_ok=True)
    (obt.path.prefix()/"sdks").mkdir(parents=True,exist_ok=True)
    (obt.path.prefix()/"tempdir").mkdir(parents=True,exist_ok=True)
    (obt.path.subspace_root()).mkdir(parents=True,exist_ok=True)
    (obt.path.quarantine()).mkdir(parents=True,exist_ok=True)
    obt.path.downloads().mkdir(parents=True,exist_ok=True)
    obt.path.builds().mkdir(parents=True,exist_ok=True)
    obt.path.manifests().mkdir(parents=True,exist_ok=True)
    obt.path.gitcache().mkdir(parents=True,exist_ok=True)
    obt.path.apps().mkdir(parents=True,exist_ok=True)
    obt.path.buildlogs().mkdir(parents=True,exist_ok=True)
  ###########################################
  def genLaunchScript(self,out_path=None,subspace=None):
    numcores = int(os.environ["OBT_NUM_CORES"])

    LAUNCHENV = []
    if self.GIT_SSH_COMMAND!=None:
      LAUNCHENV += ['export GIT_SSH_COMMAND="%s";'%self.GIT_SSH_COMMAND]
    LAUNCHENV += ["obt.env.launch.py"]
    LAUNCHENV += ["--numcores", numcores]
    LAUNCHENV += ["--launch", self.OBT_STAGE]
    LAUNCHENV += ["--prjdir", self.PROJECT_DIR]

    if subspace!= None:
      LAUNCHENV += ["--subspace", subspace]

    LAUNCHENV += [";\n"]

    f = open(str(out_path), 'w')
    f.write(" ".join(obt.command.procargs(LAUNCHENV)))
    f.close()
    os.system("chmod ugo+x %s"%str(out_path))

  ###########################################
  def genBashRc(self,out_path=None,override_sysprompt=None):
    self.log(deco.bright("Generating bashrc override_sysprompt<%s>"%override_sysprompt))
    bdeco = obt.deco.Deco(bash=True)

    HOMEDIR = obt.path.Path(os.environ["HOME"])

    BASHRC = ""

    ################################################
    # generate prompt
    ################################################

    stackindic = ""
    stacked = False
    if "OBT_STACK" in os.environ:
      stackindic = os.environ["OBT_STACK"]
      stacked = True

    ####################
    # system prompt (leftmost icon in prompt)

    SYSPROM = ""
    if obt.host.IsOsx:    
      SYSPROM = "🍎"
    elif obt.host.IsLinux:    
      SYSPROM = "🐧"

    if "OBT_USE_PROMPT_PREFIX" in os.environ:
      SYSPROM = os.environ["OBT_USE_PROMPT_PREFIX"]

    if override_sysprompt!=None:
      SYSPROM = override_sysprompt

    PROMPT = bdeco.promptL('%s[ %s %s-${OBT_SUBSPACE_PROMPT} ]'%(SYSPROM,stackindic,self.PROJECT_NAME))
    PROMPT += bdeco.promptC("\\w")
    PROMPT += bdeco.promptR("[$(parse_git_branch) ]")
    PROMPT += bdeco.bright("> ")

    ################################################
    # The sanity of this is a little debatable.
    #  on the one hand, some of the user's shell customizations are respected
    #  on the other hand this can perturb the build environment in other unexpected ways..
    ################################################

    if (os.path.exists("~/.bashrc")):
      BASHRC += 'source $HOME/.bashrc;\n' # source users's bash setup

    ################################################

    BASHRC += 'parse_git_branch() { git branch 2> /dev/null | grep "*" | sed -e "s/*//";};\n'

    BASHRC += 'export -f parse_git_branch\n'

    BASHRC += "\nexport PS1='%s';\n" % PROMPT
    BASHRC += "alias ls='ls -G';\n"
    BASHRC += "complete -r -v\n"

    #########################################
    # statically defined goto and push methods
    #########################################

    dirs = {
        "root": "${OBT_ROOT}",
        "project": "${OBT_PROJECT_DIR}",
        "deps": "${OBT_ROOT}/modules/dep",
        "subspace": "${OBT_STAGE}/subspaces/${OBT_SUBSPACE}",
        "stage": "${OBT_STAGE}",
        "builds": "${OBT_STAGE}/builds",
    }

    #########################################
    # dynamic goto and pushd methods
    #  generated from individual deps
    #########################################

    depitems = obt.dep.DepNode.FindWithMethod("env_goto")
    for depitemk in depitems:
      depitem = depitems[depitemk]
      if depitem.supports_host:
        gotos = depitem.env_goto()
        dirs.update(gotos)

    #########################################

    for k in dirs:
        v = dirs[k]
        BASHRC += "obt.goto.%s() { cd %s; };" % (k,v)
        BASHRC += "obt.push.%s() { pushd %s; };" % (k,v)

    ################################################
    # add completions.json from ~/.obt-global/completions.json
    ################################################
  
    OBT_GLOBAL = HOMEDIR/".obt-global"

    if (OBT_GLOBAL/"completions.json").exists():
      json_array = json.load(open(str(OBT_GLOBAL/"completions.json")))
      print(json_array)
      for json_item in json_array:
        BASHRC += "source %s\n" % json_item

    ################################################

    obt_completions_inp = obt.path.obt_bin_priv_base()/"_obt_dep_completions.py"
    for item in ["obt.dep.build.py","obt.dep.info.py","obt.dep.status.py"]:
      completions_line = "complete -C %s %s\n" % (str(obt_completions_inp),item)
      BASHRC += completions_line

    ################################################

    obt_completions_shell_inp = obt.path.obt_bin_priv_base()/"_obt_dep_completions_shell.py"
    completions_line = "complete -C %s obt.dep.shell.py\n" % (str(obt_completions_shell_inp))
    BASHRC += completions_line

    ################################################

    obt_completions_inp = obt.path.obt_bin_priv_base()/"_obt_subspace_completions.py"
    for item in ["obt.subspace.build.py","obt.subspace.launch.py"]:
      completions_line = "complete -C %s %s\n" % (str(obt_completions_inp),item)
      BASHRC += completions_line

    ################################################

    if out_path!=None:
      f = open(str(out_path), 'w')
      f.write(BASHRC)
      f.close()

    return BASHRC



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, inspect, tarfile, sys
from pathlib import Path
import importlib.util
import obt.path, obt.host
from obt.command import Command, run
from obt.deco import Deco
from obt.wget import wget
from obt import pathtools, cmake, make, path, git, host
from obt import _dep_impl, _dep_x, _globals, log, buildtrace
from enum import Enum

deco = Deco()

root_dep_list = ["root", "python", "pydefaults"]

class ProviderScope(Enum):
  CONTAINER = 1 # dependency is scoped to the container
  INIT = 2  # dependency is scoped to the container and rendered upon creation of container (always present)
  HOST = 3 # dependency is sourced from the host (apt install, brew install, etc...)

###############################################################################
class Provider(object):
    """base class for all dependency providers"""
    def __init__(self,name,target=None):
      self.scope = ProviderScope.CONTAINER
      self._allow_build_in_subspaces = False 
      if target ==None:
        self._target = host.description().target
      else:
        self._target = target
      self._name = name
      self._miscoptions = _globals.getOptions()
      self._node = None
      self._deps = {}
      self._archlist = None 
      self._oslist = None 
      self._src_required_files = []
      self._bin_required_files = []
      self._required_deps = {}
      self._topoindex = -1
      self.manifest = path.manifests()/name
      self.OK = self.manifest.exists()
      self.setSourceRoot(path.builds()/name)
      self._debug = False
      self._must_build_in_tree = False
      if name not in root_dep_list:
        self.declareDep("root")
    #############################
    def declareDep(self, named):
      inst = _dep_x.instance(named)
      #print(named,type(inst))
      self._required_deps[named] = inst
      return inst
    #############################
    def createBuilder(self,clazz, **kwargs):

      if len(kwargs)>0:
        self._builder = clazz(self._name,**kwargs)
      else:
        self._builder = clazz(self._name)
      if len(self._required_deps):
        self._builder.requires(self._required_deps)
      return self._builder
    #############################
    def setSourceRoot(self,srcroot):
      self.source_root = srcroot
      self.build_src = srcroot
      self.build_dest = srcroot/".build"
      self.build_working_dir = self.build_dest
    #############################
    def mustBuildInTree(self):
      self.build_dest = self.build_src
      self._must_build_in_tree = True
    #########################################
    def areRequiredSourceFilesPresent(self):
      return None
    #########################################
    def areRequiredBinaryFilesPresent(self):
      return self.areRequiredSourceFilesPresent()
    #############################
    @property
    def scopestr(self):
      return str(self.scope).split('.')[1]
    #############################
    ## wipe build ?
    #############################
    @property
    def supports_host(self):
      """predicate determining if dependency supports the host architecture and OS"""
      supports = False 
      #####################################
      def check_os():
        check = False
        if self._oslist==None:
          check = True
        elif host.IsDarwin and ("Darwin" in self._oslist):
          check = True
        elif host.IsLinux and ("Linux" in self._oslist):
          check = True
        elif host.IsIrix and ("IRIX64" in self._oslist):
          check = True
        return check
      #####################################
      def check_arch():
        check = False
        if self._archlist==None:
          check = True
        elif host.IsX86_64 and ("x86_64" in self._archlist):
          check = True
        elif host.IsAARCH64 and ("aarch64" in self._archlist):
          check = True
        return check
      #####################################
      return check_os() and check_arch()

    #############################
    ## Is this the primary dep provvider?
    #############################
    @property
    def is_primary_dep(self):
      if "depname" in self._miscoptions:
        if self._name == self._miscoptions["depname"]:
          return True 
      return False  
    #############################
    ## wipe build ?
    #############################

    @property
    def should_wipe(self):
      """predicate controlling build-clean during provisioning of a dependency"""
      wipe = False
      if "wipe" in self._miscoptions:
        wipe = self._miscoptions["wipe"]==True
      return wipe and self.is_primary_dep

    #############################
    ## serial build ?
    #############################

    @property
    def should_serial_build(self):
      """predicate controlling 1cpu build during provisioning of a dependency (for build debugging)"""
      serial = False
      if "serial" in self._miscoptions:
        serial = self._miscoptions["serial"]==True
      return serial

    @property
    def default_parallelism(self):
      parallelism = 1.0
      if self.should_serial_build:
        parallelism=0.0
      return parallelism

    #############################
    ## force build ?
    #############################

    @property
    def should_force_build(self):
        force = False
        if "force" in self._miscoptions:
          force = self._miscoptions["force"]==True
        return force and self.is_primary_dep

    #############################
    ## force build ?
    #############################

    @property
    def should_incremental_build(self):
        incremental = False
        if "incremental" in self._miscoptions:
          incremental = self._miscoptions["incremental"]==True
        return incremental and self.is_primary_dep

    #############################

    @property
    def should_build(self):
        return (not self.manifest.exists()) or self.should_force_build or self.should_incremental_build

    #############################

    @property
    def should_fetch(self):
        fetch = False
        if "nofetch" in self._miscoptions:
          fetch = self._miscoptions["nofetch"]==False
        return fetch

    #############################

    @property
    def _std_cmake_vars():
      cmakeEnv = {
        "CMAKE_BUILD_TYPE": "Release",
        "BUILD_SHARED_LIBS": "ON",
      }
      return cmakeEnv

    #############################

    @property
    def exists(self):
        return False

    #############################

    def option(self,named):
      if named in self._miscoptions:
        return self._miscoptions[named]
      else:
        return None

    #############################

    def provide(self):
      if not self.supports_host:
        print(deco.red("Dependency does not support this host"))
        return False
      if self.should_wipe:
        self.wipe()
      if self.should_build:
        with buildtrace.NestedBuildTrace({ "op": "Provider.provide(%s)"%self._name }) as nested:
          self.OK = self.build()
          if self.OK:
            self.OK = self.onPostBuild()
            if self.OK:
              gfnkdsgnjf=self.areRequiredBinaryFilesPresent()
              if gfnkdsgnjf != None:
                self.OK = gfnkdsgnjf
              else:
                self.OK = True
              if self.OK:
                self.manifest.touch()
      return self.OK


    #############################

    def onPostBuild(self):
      return True

    #############################

    def _std_cmake_build(self,srcdir,blddir,cmakeEnv=_std_cmake_vars,parallelism=1.0):
      ok2build = True
      if self.should_incremental_build:
        os.chdir(blddir)
      else:
        pathtools.mkdir(blddir,clean=True)
        pathtools.chdir(blddir)
        cmake_ctx = cmake.context(root=srcdir,env=cmakeEnv)
        ok2build = cmake_ctx.exec()==0
      if ok2build:
        return (make.exec("install",parallelism=parallelism)==0)
      return False

    #############################

    def wipe(self):
        pass

    #############################

    def node(self):
        return self._node

    #############################

    @property
    def shlib_extension(self):
      if host.IsOsx:
        return "dylib"
      if host.IsIx:
        return "so"

###############################################################################

class HomebrewProvider(Provider):
  def __init__(self,name,pkgname):
    super().__init__(pkgname)
    self.manifest = path.manifests()/name
    self.OK = self.manifest.exists()
    self.pkgname = pkgname
    self._deps = list()
  ###########################################
  def requires(self,deplist):
    self._deps += deplist
  ###########################################
  def brew_prefix(self):
    return Path("/")/"usr"/"local"
  ###########################################
  def build(self):
    for item in self._deps:
      n = _dep_x.instance(item)
      print(item,n)
      assert(n!=None)
      n.provide()
    retc = Command(["brew","install",self.pkgname]).exec()
    if 0 == retc:
      self.manifest.touch()
    return 0==retc

###############################################################################

class StdProvider(Provider):
    #############################
    def __init__(self,name):
      super().__init__(name)
      self._builder = None

    ########

    @property
    def _fetcher(self):
      return None

    #############################
    def postinit(self):
      pass
    #############################
    def requires(self,deplist):
      self._builder.requires(deplist)
    #############################
    def install(self):
      return self._builder.install(self.build_dest)
    #############################
    def __str__(self):
      if hasattr(self,"descriptor"):
        return self.descriptor()
      else:
        return self._fetcher.descriptor()
    #############################
    def wipe(self):
      os.system("rm -rf %s"%self.source_root)
    #############################
    def updateSource(self):
        return self._fetcher.update(self.source_root)
    #############################
    def _fetch(self):
      source_exists = self.areRequiredSourceFilesPresent()
      #print(deco.bright("STDPROVIDER<%s> source_exists<%s>"%(self._name,source_exists)))
      fetchOK = False
      if not source_exists:
        print(deco.bright("Fetching<%s>"%(self._name)))
        fetchOK = self._fetcher.fetch(self.source_root)
        #assert(fetchOK==True or fetchOK==False)
        if fetchOK:
          self._fetcher.patch()
        else:
          print(deco.err("Fetch <%s> failed!"%self._name))
          fetchOK = False
      return fetchOK 
    #############################
    def build(self):
      if self.areRequiredSourceFilesPresent() == False:
        print(deco.red("Cannot build <%s> missing files[%s]"%(self._name,self._src_required_files)))
        return False
      #########################################
      # build
      #########################################
      print(deco.bright("Building<%s>"%(self._name)))
      self.OK = self._builder.build(self.build_src,
                                    self.build_dest,
                                    self.build_working_dir,
                                    self.should_incremental_build)
      #########################################
      if not self.OK:
        print(deco.err("Build <%s> failed!"%self._name))
      return self.OK
    #########################################
    def install(self):
      return self._builder.install(self.build_dest)
    #########################################
    def provide(self):
      with buildtrace.NestedBuildTrace({ "op": "StdProvider.provide(%s)"%self._name }) as nested:
       #print("self.should_wipe<%d>"%self.should_wipe)
       #print("self.should_build<%d>"%self.should_build)
       self.postinit()

       #########################################
       # WIPE
       #########################################

       if self.should_wipe:
         self.wipe()

       #########################################
       # FETCH
       #########################################

       src_present = self.areRequiredSourceFilesPresent()
      
       if not src_present:
         fetch_ok = self._fetch()
         if False==fetch_ok:
          print(deco.err("Fetch <%s> failed!"%self._name))
          self.OK = False
          return False

       #########################################
       # BUILD
       #########################################

       if self.should_build:
        self.OK = self.build()

        if self.OK:
          self.OK = self.onPostBuild()

          #########################################
          # INSTALL
          #########################################

          if self.OK:
            self.OK = self.install()
          else:
            return False

       #########################################
       # MANIFEST
       #########################################

       if self.OK:
        self.manifest.touch()
      
       assert(self.OK)
      
       return self.OK
    #########################################




#!/usr/bin/env python3

import os

deplist =  ["cmake","wget","curl","libtiff","libpng", "git-lfs"]
deplist += ["portaudio","audiofile","m4","bison","flex","xz"]
deplist += ["scons","zlib","tbb", "glew","boost","flac","libsndfile"]
deplist += ["mpfr","openssl","graphviz","doxygen","swig","tcl-tk"]

depliststr = " ".join(deplist)
os.system("brew install %s" % depliststr)

os.system("/usr/local/bin/python3 -m pip install --upgrade setuptools")



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt.command import Command
import obt.host
import obt.math

def exec(target=None,parallelism=1.0):
  """
  Execute make with target and specific parallelism.
   Assumes Makefile present in current directory.
  Keyword Arguments:
  target - makefile target (eg all, default, install, etc..)
  parallelism - numjobs normalized to numcores/2 (0.0: numjobs=1, 1.0: numjobs=numcores/2)
  """
  cmd = ["make"]
  print("make with parallel<%g>"%parallelism)
  if parallelism!=0.0:
    p = obt.math.clamp(parallelism,0.0,1.0)
    numcores = int(obt.host.NumCores*p)
    numcores = obt.math.clamp(numcores,1,obt.host.NumCores)
    cmd += ["-j",numcores]
  if target!=None:
    cmd += [target]
  return Command(cmd).exec()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "4.1.2"
HASH = "3a4621dd15a774658008e43203f49137"

import os, tarfile
from yarl import URL
from obt import dep, host, path, cmake
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command

deco = Deco()

###############################################################################

class wt4(dep.Provider):

  def __init__(self): ############################################
    super().__init__("wt4")
    #print(options)
    build_dest = path.builds()/"wt4"
    self.build_dest = build_dest
    self.manifest = path.manifests()/"wt4"
    self.OK = self.manifest.exists()
    self.fname = "wt-%s.tar.gz"%VERSION
    self._archlist = ["x86_64"]

  ########

  def __str__(self):
    return "WT4 (github-%s)" % VERSION

  ########

  def download_and_extract(self): #############################################

    url = URL("https://github.com/emweb/wt/archive/%s/.tar.gz"%VERSION)

    self.arcpath = dep.downloadAndExtract([url],
                                          self.fname,
                                          "gz",
                                          HASH,
                                          self.build_dest)


  def build(self): ############################################################

    #psql = dep.require("postgresql").instance

    source_dir = self.build_dest/("wt-%s"%VERSION)
    build_temp = source_dir/".build"
    print(build_temp)
    if build_temp.exists():
      Command(["rm","-rf",build_temp]).exec()

    build_temp.mkdir(parents=True,exist_ok=True)
    os.chdir(str(build_temp))
    cmakeEnv = {
      "BOOST_ALL_DYN_LINK": None,
      "WT_BOOST_DISCOVERY": 1,
      #"BOOST_COMPILER": "clang++",
      #"BOOST_PREFIX": path.prefix(),
      #"BOOST_VERSION": boost.version,
      "ENABLE_MYSQL": False,
      #"ENABLE_POSTGRES": True,
      "ENABLE_QT4": False,
      "ENABLE_SQLITE": False,
      "WT_CPP_11_MODE": "-std=c++11",
    }
    if host.IsOsx:
      cmakeEnv["CMAKE_MACOSX_RPATH"]=1
      cmakeEnv["CMAKE_INSTALL_RPATH"]=path.prefix()/"lib"

    cmake.context(root=source_dir,env=cmakeEnv).exec()
    return 0==Command(["make","-j",host.NumCores,"install"]).exec()

  def provide(self): ##########################################################

    if False==self.OK:
      self.download_and_extract()
      self.OK = self.build()
      if self.OK:
        self.manifest.touch()
    return self.OK



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile
from obt import dep, host, path, pathtools
from obt.deco import Deco
from obt.wget import wget
import obt.git
from obt.command import Command
import obt.patch
import obt.pip

deco = Deco()

###########################################################

class pillar(dep.Provider):

  ################################

  def __init__(self):
    super().__init__("pillar")
    self.source_root = path.builds()/"pillar-python-sdk"
    self.url = "https://github.com/armadillica/pillar"
    self.manifest = path.manifests()/"pillar-python-sdk"
    self.OK = self.manifest.exists()
    if self.option("force")==True:
      self.OK = False

  ################################

  def __str__(self):
    return "pillar-python-sdk (%s)" % self.url

  ################################

  def provide(self):

    if False==self.OK:
      obt.git.Clone(self.url,self.source_root,"master")
      os.chdir(self.source_root)

      # install deps
      obt.pip.install([ "raven",
                        "celery",
                        "bleach",
                        "CommonMark",
                        "Flask-Babel"])

      # patch pillar

      patcher = obt.patch.patcher("pillar")
      patcher.patch_list([[self.source_root/"pillar","markdown.py"]])

      # install

      Command(["python3","setup.py", "install"]).exec()

      self.manifest.touch()

    return True



###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

class targetinfo:
 def __init__(self):
   self.architecture = "aarch64"
   self.os = "android"
   self.c_compiler = "clang"
   self.cxx_compiler = "clang++"
   self.identifier = "aarch64-android"


from ork import dep, path, command, docker, wget
from ork.deco import Deco
import ork.module
import time, re, socket, os, sys
from pathlib import Path
deco = ork.deco.Deco()

this_path = os.path.realpath(__file__)
this_dir = Path(os.path.dirname(this_path))

###############################################################################

VERSION = "9.6"
MD5SUM = "bd07b8596c44133dd6be58b55dccca16"

###############################################################################

class dockerinfo:
    ###############################################
    def __init__(self):
      super().__init__()
      self.type = docker.Type.SINGLE
      self._name = "sagemath"
      self.containername = "obt-sagemath"
      self.imagename = "obt/sagemath-jupyter"
      self.tryipaddr = socket.gethostbyname(socket.gethostname())
      self.archivename = "sage-%s.tar.gz" % VERSION
    ###############################################
    # build the docker image
    ###############################################
    def build(self,build_args):
      os.chdir(path.stage())
      URL = "http://sage.mirror.garr.it/mirrors/sage/src/%s"%self.archivename
      _archive_path = wget.wget(urls=[URL],
                       md5val=MD5SUM,
                       output_name=self.archivename)
      print(_archive_path)
      if _archive_path!=None:
        command.run([
          "docker", "build", "-f", this_dir/"Dockerfile", "-t", self.imagename, "." ] )
    ###############################################
    # kill active docker container
    ###############################################
    def kill(self):
        os.chdir(this_dir)
        retstr = command.capture([
            "docker-compose",
            "-f","docker-compose.yml",
            "down"
        ])
    ###############################################
    # launch docker container
    #  print out connection info
    ###############################################
    def launch(self,launch_args):
        #self.kill()
        os.chdir(this_dir)
        command.run([
            "docker-compose",
            "-f","docker-compose.yml",
            "up","--remove-orphans",
            "--detach"
        ])
        time.sleep(2)
        command.run([
            "docker-compose","logs"
        ])
        ######################
        #print("%s<%s>"%(deco.key("URL"),deco.val(url)))
        #print("%s<%s>"%(deco.key("TOKEN"),deco.val(token)))
    ###############################################
    # information dictionary
    ###############################################
    def info(self):
      return {
        "container_name": self.containername,
        "image_name": self.imagename,
        "version": VERSION
      }
    ###############################################